import os
from django.conf import settings

BROWSCAP_PATH = getattr(settings, 'BROWSCAP_PATH', None)
if BROWSCAP_PATH is None:
    this_dir = os.path.dirname(os.path.abspath(__file__))
    BROWSCAP_PATH = os.path.join(this_dir, 'browscap.ini')

MOBILE_DOMAIN = getattr(settings, 'MOBILE_DOMAIN', 'http://m.example.com')
MOBLER_CACHE_KEY = getattr(settings, 'MOBLER_CACHE_KEY', 'mobler-cache')
MOBLER_CACHE_TIMEOUT = getattr(settings, 'MOBLER_CACHE_TIMEOUT', 60*60*1)
MOBLER_COOKIE_NAME = getattr(settings, 'MOBLER_COOKIE_NAME', 'mobler-override')
MOBLER_COOKIE_AGE = getattr(settings, 'MOBLER_COOKIE_AGE', 60*60*24*31)
MOBLER_PRESERVE_URL = getattr(settings, 'MOBLER_PRESERVE_URL', True)




from django.shortcuts import redirect
from mobler import settings as mobler_settings
from mobler.browscap import browser
import time
import urlparse
import urllib

COOKIE_AGE = mobler_settings.MOBLER_COOKIE_AGE
COOKIE_NAME = mobler_settings.MOBLER_COOKIE_NAME

def build_full_path(request):
    ''' Build a full path+querystring (sans Mobler override) from a request '''
    path = request.path
    if request.GET:
        params = request.GET.copy()
        if COOKIE_NAME in params:
            del params[COOKIE_NAME]
        path += '?%s' % urllib.urlencode(params)
    return path

class MobileDetectionMiddleware(object):
    def process_request(self, request):
        user_agent = request.META.get('HTTP_USER_AGENT', '')
        is_mobile = browser.detect_mobile(user_agent)
        do_override = (request.COOKIES.get(COOKIE_NAME) == '1')

        ## Check for the override GET param. If we have it, set the cookie and refresh.
        if is_mobile and request.GET.get(COOKIE_NAME):
            response = redirect(build_full_path(request))
            response.set_cookie(COOKIE_NAME, '1', time.time()+COOKIE_AGE)
            return response

        ## If we have a mobile UA but no browser override, redirect
        ## to the mobile site.
        if is_mobile and not do_override:
            mobile_url = mobler_settings.MOBILE_DOMAIN
            if mobler_settings.MOBLER_PRESERVE_URL:
                mobile_url = urlparse.urljoin(mobile_url, build_full_path(request))
            return redirect(mobile_url)

        ## We might be mobile, so set a flag on the request for convenience.
        request.is_mobile = is_mobile



from django.test import TestCase
from django.test.client import Client
from mobler import settings as mobler_settings
from mobler.browscap import browser
import urlparse

TEST_AGENTS = {
    ## is_mobile: ua_list
    True: (
        'Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1',
        'Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1A543a Safari/419.3'
        ),
    False: (
        'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1',
        'Mozilla/5.0 (Windows NT 6.1; U; ru; rv:5.0.1.6) Gecko/20110501 Firefox/5.0.1 Firefox/5.0.1',
        ),
}

MOBILE_UA = TEST_AGENTS[True][0]
BROWSER_UA = TEST_AGENTS[False][0]
COOKIE_NAME = mobler_settings.MOBLER_COOKIE_NAME

class MoblerTests(TestCase):
    def test_browscap(self):
        ''' Test that browscap is properly parsing user agents '''
        for is_mobile, ua_list in TEST_AGENTS.iteritems():
            for ua_string in ua_list:
                self.assertEqual(is_mobile, browser.detect_mobile(ua_string))

    def test_mobile_redirect(self):
        ''' Test that mobile UA's get redirected and browser UA's don't '''
        ## Mobile UA's should be redirected
        response = self.client.get('/', {}, HTTP_USER_AGENT=MOBILE_UA)
        self.assertEqual(response.status_code, 302)

        ## Browser UA's should *not* be redirected
        response = self.client.get('/', {}, HTTP_USER_AGENT=BROWSER_UA)
        self.assertNotEqual(response.status_code, 302)

    def test_override(self):
        ''' Test that mobile UA's with the override GET parameter are not redirected '''
        ## Test that setting the override param results in a redirect
        response = self.client.get('/', {COOKIE_NAME: '1'}, HTTP_USER_AGENT=MOBILE_UA)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(self.client.cookies[COOKIE_NAME].value, '1')

        ## With the cookie set, make sure future requests are not redirected.
        response = self.client.get('/', {}, HTTP_USER_AGENT=MOBILE_UA)
        self.assertNotEqual(response.status_code, 302)
        self.assertEqual(self.client.cookies[COOKIE_NAME].value, '1')

    def test_url_preserved(self):
        ''' Test that the URL is preserved when redirecting to the mobile site '''
        mobler_settings.MOBLER_PRESERVE_URL = True
        response = self.client.get('/foo/bar/?fizz=buzz', {}, HTTP_USER_AGENT=MOBILE_UA)
        redirect_url = urlparse.urlparse(response._headers['location'][1])
        self.assertEqual(redirect_url.path, '/foo/bar/')
        self.assertEqual(redirect_url.query, 'fizz=buzz')



import urllib
from django import template
from mobler import settings as mobler_settings

register = template.Library()
COOKIE_NAME = mobler_settings.MOBLER_COOKIE_NAME

@register.simple_tag
def full_site_override(request):
    path = request.path
    params = request.GET.copy()
    params.setdefault(COOKIE_NAME, '1')
    return '%s?%s' % (path, urllib.urlencode(params))



#!/usr/bin/env python

import os
from mobler import __version__, __description__
from setuptools import setup, find_packages

## Package detection from django-registration

# Compile the list of packages available, because distutils doesn't have
# an easy way to do this.
packages, data_files = [], []
root_dir = os.path.dirname(__file__)
if root_dir:
    os.chdir(root_dir)

for dirpath, dirnames, filenames in os.walk('mobler'):
    # Ignore dirnames that start with '.'
    for i, dirname in enumerate(dirnames):
        if dirname.startswith('.'): del dirnames[i]
    if '__init__.py' in filenames:
        pkg = dirpath.replace(os.path.sep, '.')
        if os.path.altsep:
            pkg = pkg.replace(os.path.altsep, '.')
        packages.append(pkg)
    elif filenames:
        prefix = dirpath[11:] # Strip "mobler/" or "mobler\"
        for f in filenames:
            data_files.append(os.path.join(prefix, f))

setup(
    name='django-mobler',
    version=__version__,
    description=__description__,
    long_description=__description__,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        ],
    keywords='django mobile browscap',
    maintainer = 'Derek Payton',
    maintainer_email = 'derek.payton@gmail.com',
    url='https://github.com/dmpayton/django-mobler',
    download_url='https://github.com/dmpayton/django-mobler/tarball/v1.0.0',
    license='MIT License',
    include_package_data=True,
    packages=packages,
    package_data={'mobler': data_files},
    package_dir={'mobler': 'mobler'},
    zip_safe=False,
    )



import os
import re
from ConfigParser import SafeConfigParser
from django.core.cache import cache
from mobler import settings as mobler_settings

CACHE_KEY = mobler_settings.MOBLER_CACHE_KEY
CACHE_TIMEOUT = mobler_settings.MOBLER_CACHE_TIMEOUT

class BrowscapParser(object):
    def __init__(self):
        self.mobile_browsers = cache.get(CACHE_KEY, [])
        self._cache = {}
        if not self.mobile_browsers:
            config = SafeConfigParser()
            config.read(mobler_settings.BROWSCAP_PATH)

            browsers = {}
            parents = set()

            for name in config.sections():
                data = dict(config.items(name))
                parent = data.get('parent')
                if parent:
                    parents.add(parent)
                browsers[name] = data

            for name, data in browsers.iteritems():
                if name in parents:
                    continue

                parent = data.get('parent')
                if parent:
                    data.update(browsers[parent])

                if data.get('ismobiledevice') == 'true':
                    name = re.escape(name)
                    name = name.replace("\\?", ".").replace("\\*", ".*?")
                    name = "^%s$" % name
                    self.mobile_browsers.append(name)
            self.mobile_browsers = map(re.compile, self.mobile_browsers)
            cache.set(CACHE_KEY, self.mobile_browsers, CACHE_TIMEOUT)

    def detect_mobile(self, user_agent):
        ''' Detect whether a given User-Agent is a mobile device. '''
        ## If we've cached this UA, return that value
        try:
            return self._cache[user_agent]
        except KeyError:
            ## No match, we'll have to look it up.
            pass

        ## Iterate through our list of regexps to determin if this UA is mobile
        for ua in self.mobile_browsers:
            if ua.match(user_agent):
                ## We have a match!
                self._cache[user_agent] = True
                break
        else:
            ## No match found
            self._cache[user_agent] = False

        return self._cache[user_agent]

browser = BrowscapParser()
